package sock

import (
	"fmt"
	"io"
	"net"
	"os"

	"github.com/jezek/xgb"
)

const SOCKET_PATH_TPL = "/tmp/bspwm%s_%d_%d-socket"

type Sock struct {
	C net.Conn
}

func (s *Sock) Send(args ...string) (string, error) {
	if len(args) == 0 {
		return "", s.C.Close()
	}
	for _, msg := range args {
		_, err := s.C.Write([]byte(msg))
		if err != nil {
			return "", err
		}
		_, err = s.C.Write([]byte{0})
		if err != nil {
			return "", err
		}
	}
	bts, err := io.ReadAll(s.C)
	if err != nil {
		return "", err
	}
	if len(bts) > 0 {
		if bts[0] == 7 {
			return "", fmt.Errorf(string(bts[1:]))
		}
	}
	return string(bts), nil
}

func Default() (*Sock, error) {
	return New(os.Getenv("BSPWM_SOCKET"))
}
func New(path string) (*Sock, error) {
	xc, err := xgb.NewConn()
	if err != nil {
		return nil, err
	}
	defer xc.Close()
	if path == "" {
		path = fmt.Sprintf(SOCKET_PATH_TPL, "", xc.DisplayNumber, xc.DefaultScreen)
	}
	conn, err := net.Dial("unix", path)
	if err != nil {
		return nil, err
	}
	return &Sock{C: conn}, nil
}
